-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module	= ElvUI_ChatTweaks:NewModule("Pet Battles", "AceConsole-3.0", "AceEvent-3.0")
local L			= ElvUI_ChatTweaks.L
Module.name		= L["Pet Battles"]

local db, options
local defaults = {
	profile = {
		stableMaster	= true,
		battle			= {
			enable	= true,
			color	= {r = 0.72, g = 0.63, b = 0.31},	-- tannish/yellowish
			frame	= "ChatFrame3",
		},
		combat			= {
			enable	= true,
			color	= {r = 0.09, g = 0.52, b = 0.82},	-- bluish
			frame	= "ChatFrame11",
		},
	}
}

local battleMSG		= ""
local combatMSG		= ""
local frameString	= "ChatFrame%d"
local frameList		= L["Chat Frame %d"]

local function GetStableMasterIndex()
	for i = 1, GetNumTrackingTypes() do
		local name, _, active, _ = GetTrackingInfo(i)
		if name == MINIMAP_TRACKING_STABLEMASTER then
			return i, active
		end
	end
end

local function TrackStableMaster(enabled)
	local index, _ = GetStableMasterIndex()
	SetTracking(index, enabled)
end

local function PetBattleFilter(self, event, message)
	if not db.battle.enable or not message or not _G[db.battle.frame] then return false end
	if Module.debug then
		_G[db.battle.frame]:AddMessage("I'm using this ChatFrame.")
		print(message)
	end
	
	if message ~= battleMSG then	-- prevent duplicate messages
		_G[db.battle.frame]:AddMessage(message, db.battle.color.r, db.battle.color.g, db.battle.color.b)
		battleMSG = message
	end
	return true
end

local function PetCombatFilter(self, event, message)
	if not db.combat.enable or not message or not _G[db.combat.frame] then return false end
	if Module.debug then
		_G[db.combat.frame]:AddMessage("I'm using this ChatFrame.")
		print(message)
	end
	
	if message ~= combatMSG then
		_G[db.combat.frame]:AddMessage(message, db.combat.color.r, db.combat.color.g, db.combat.color.b)
		combatMSG = message
	end
	return true
end

function Module:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_PET_BATTLE_INFO", PetBattleFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_PET_BATTLE_COMBAT_LOG", PetCombatFilter)
	
	if db.stableMaster then
		local _, active = GetStableMasterIndex()
		if not active then TrackStableMaster(true) end
	end
end

function Module:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_PET_BATTLE_INFO", PetBattleFilter)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_PET_BATTLE_COMBAT_LOG", PetCombatFilter)
	
	if db.stableMaster then
		local _, active = GetStableMasterIndex()
		if active then TrackStableMaster(false) end
	end
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("PetBattles", defaults)
	db = self.db.profile
	Module.debug = ElvUI_ChatTweaks.db.profile.debugging
end

function Module:GetOptions()
	if not options then
		options = {
			stableMaster	= {
				type	= "toggle",
				order	= 13,
				name	= L["Track Stable Masters"],
				desc	= L["Automatically enable tracking of Stable Masters to help you revive and/or heal your pets."],
				get		= function() return db.stableMaster end,
				set		= function(_, value)
					local index, _ = GetStableMasterIndex()
					db.stableMaster = value 
					SetTracking(index, value)
				end,
			},
			battle = {
				type		= "group",
				guiInline	= true,
				order		= 14,
				name		= L["Pet Battle Info"],
				args		= {
					enable = {
						type		= "toggle",
						order		= 1,
						name		= L["Enable Filtering"],
						desc		= L["Toggle coloring and changing of the chat frame for pet battle info messages."],
						get			= function() return db.battle.enable end,
						set			= function(_, value) db.battle.enable = value end,
					},
					color = {
						type		= "color",
						order		= 2,
						name		= L["Message Color"],
						disabled	= function() return not db.battle.enable end,
						get			= function() return db.battle.color.r, db.battle.color.g, db.battle.color.b end,
						set			= function(_, r, g, b) db.battle.color.r = r; db.battle.color.g = g; db.battle.color.b = b; end,
					},
					frame = {
						type		= "select",
						order		= 3,
						name		= L["Output Frame"],
						desc		= L["Chat frame to route the messages to."],
						disabled	= function() return not db.battle.enable end,
						get			= function() return db.battle.frame end,
						set			= function(_, value) db.battle.frame = value end,
						values		= function()
							local frames = {}
							for i = 1, 18 do
								if _G[frameString:format(i)] then
									frames[frameString:format(i)] = frameList:format(i)
								end
							end
							return frames
						end,
					},
				},
			},
			combat = {
				type		= "group",
				guiInline	= true,
				order		= 15,
				name		= L["Pet Combat Log"],
				args		= {
					enable = {
						type		= "toggle",
						order		= 1,
						name		= L["Enable Filtering"],
						desc		= L["Toggle coloring and change of the chat frame for pet combat log messages."],
						get			= function() return db.combat.enable end,
						set			= function(_, value) db.combat.enable = value end,
					},
					color = {
						type		= "color",
						order		= 2,
						name		= L["Message Color"],
						disabled	= function() return not db.combat.enable end,
						get			= function() return db.combat.color.r, db.combat.color.g, db.combat.color.b end,
						set			= function(_, r, g, b) db.combat.color.r = r; db.combat.color.g = g; db.combat.color.b = b; end,
					},
					frame = {
						type		= "select",
						order		= 3,
						name		= L["Output Frame"],
						desc		= L["Chat frame to route the messages to."],
						disabled	= function() return not db.combat.enable end,
						get			= function() return db.combat.frame end,
						set			= function(_, value) db.combat.frame = value end,
						values		= function()
							local frames = {}
							for i = 1, 18 do
								if _G[frameString:format(i)] then
									frames[frameString:format(i)] = frameList:format(i)
								end
							end
							return frames
						end,
					},
				},
			},
		}
	end
	return options
end

function Module:Info()
	return L["Allows you to color the pet battle info messages, and determine which chat frame to send them to."]
end